import React, {Fragment, useEffect} from 'react'
import {
    Form,
    Input,
    Button,
    Dialog,
    TextArea,
    Space, Tag, Radio, Checkbox
} from 'antd-mobile'
import ParentTask from "../ParentTask";
import "./index.css"
import {addTask, getPTask, getTaskById, updateTask} from "../../utils";
import {useLocation, useNavigate, useOutletContext, useSearchParams} from "react-router-dom";
import dayjs from "dayjs";
import DataPickItemPopup from "../DataPickItemPopup";
import {getDictionary} from "../../utils/dictUtil";

const DetailForm = () => {
    // 进入此页面的操作：添加，修改，详情（按钮为添加任务日志）
    const location = useLocation();
    let [params] = useSearchParams();
    // 设置标题栏
    const {setTitle, setRightDesc} = useOutletContext();
    const [currentPath, setCurrentPath] = React.useState("");
    const [updateFiledDisabled, setUpdateFiledDisabled] = React.useState(true);
    const [pName, setPName] = React.useState();
    const [pidArray, setPidArray] = React.useState([]);
    const [stateList, setStateList] = React.useState([]);
    const [priorityList, setPriorityList] = React.useState([]);
    // 路由
    const navigate = useNavigate();
    // 获取form引用
    const [form] = Form.useForm();
    const addEditPName = (name) => {
        setPName(name)
    }
    useEffect(() => {
        getDictionary("2").then(stateDictionary => {
            setStateList(Array.from(stateDictionary.values()));
        })
        getDictionary("1").then(priorityDictionary => {
            setPriorityList(Array.from(priorityDictionary.values()));
        })
        window.scrollTo(0, 0);
        if (location.pathname.endsWith("addTask")) {
            setTitle("添加任务");
            setCurrentPath("addTask");
            setUpdateFiledDisabled(false);
            setRightDesc(<Fragment/>)
        } else if (location.pathname.endsWith("updateTask")) {
            setTitle("修改任务");
            setCurrentPath("updateTask");
            initData(params.get('id'));
            setUpdateFiledDisabled(false);
            setRightDesc(<Fragment/>)
        } else if (location.pathname.endsWith("selectTask")) {
            setTitle("任务详情");
            setCurrentPath("selectTask");
            initData(params.get('id'));
            setUpdateFiledDisabled(true);
            setRightDesc(<div>
                <Button type="button" color="danger" onClick={() => {
                    Dialog.show({
                        content: `进入任务编辑`,
                        closeOnAction: true,
                        actions: [
                            [
                                {
                                    key: 'cancel',
                                    text: '取消',
                                },
                                {
                                    key: 'confirm',
                                    text: '确认',
                                    onClick: () => {
                                        navigate(`/detail/updateTask?id=${params.get('id')}`)
                                    }
                                }
                            ],
                        ],
                    })
                }}>编辑</Button>
            </div>)
        } else {
            // todo 异常处理
        }
    }, [location])

    function editParentTask(id) {
        if (id === '0') {
            return;
        }
        // 获取父任务信息
        getPTask(id).then(res => {
            let parentMessageVOList = res[0].parentMessageVOList;
            console.log({res, parentMessageVOList});
            setPName(parentMessageVOList[parentMessageVOList.length - 1].name);
            setPidArray(parentMessageVOList.map(parent => parent.id))
            form.setFieldValue("pidArray", parentMessageVOList.map(parent => parent.id))
        })
    }

    const initData = (id) => {
        if (id) {
            getTaskById(id).then((res) => {
                let contentElement = res.content[0];
                if (contentElement) {
                    console.log(contentElement.expectedStartTime, contentElement.expectedEndTime,
                        dayjs(contentElement.expectedStartTime).format(), dayjs(contentElement.expectedStartTime).toDate())
                    contentElement.expectedStartTime && (contentElement.expectedStartTime = dayjs(contentElement.expectedStartTime).toDate())
                    contentElement.expectedEndTime && (contentElement.expectedEndTime = dayjs(contentElement.expectedEndTime).toDate())
                    contentElement.actualStartTime && (contentElement.actualStartTime = dayjs(contentElement.actualStartTime).toDate())
                    contentElement.actualEndTime && (contentElement.actualEndTime = dayjs(contentElement.actualEndTime).toDate())
                    form.setFieldsValue(contentElement)
                    editParentTask(contentElement.pid)
                }
            })
        }
    }
    const onFinish = (values) => {
        if (currentPath === "selectTask") {
            // 进入添加日志页面
            navigate(`/detail/logTask?id=${params.get('id')}`)
            return;
        }
        console.log("提交：", values)
        if (values.pidArray?.length > 0) {
            values.pid = values.pidArray[values.pidArray.length - 1];
        } else if (!values.pid) {
            values.pid = '0'
        }
        if (currentPath === "addTask") {
            addTask(values).then(values => {
                Dialog.show({
                    content: `添加任务${values}成功`,
                    closeOnAction: true,
                    actions: [
                        [
                            {
                                key: 'back',
                                text: '回到列表',
                                onClick: () => {
                                    navigate("/home/listTask")
                                }
                            },
                            {
                                key: 'create',
                                text: '创建下一条',
                                onClick: () => {
                                    // 清空值
                                    form.resetFields();
                                    form.setFieldValue("pidArray", values.pidArray);
                                    setPidArray(values.pidArray)
                                    window.scrollTo(0, 0);
                                }
                            },
                        ],
                    ],
                })
            })
            return;
        }
        if (currentPath === "updateTask") {
            updateTask(values).then(values => {
                Dialog.show({
                    content: `修改任务${values}成功`,
                    closeOnAction: true,
                    actions: [
                        [
                            {
                                key: 'back',
                                text: '回到列表',
                                onClick: () => {
                                    navigate("/home/listTask")
                                }
                            }
                        ],
                    ],
                })
            })
        }
    }
    return (
        <>
            <Form
                form={form}
                layout='horizontal'
                onFinish={onFinish}
                footer={
                    <Button block type='submit' color='primary' size='large'>
                        {currentPath === "selectTask" ? "日记/心得" : "提交"}
                    </Button>
                }
            >
                <Form.Item
                    key='id'
                    name='id'
                    hidden={true}
                >
                </Form.Item>
                <Form.Item
                    key='pid'
                    name='pid'
                    hidden={true}
                >
                </Form.Item>
                <Form.Item
                    key='code'
                    name='code'
                    hidden={true}
                ></Form.Item>
                <ParentTask pName={pName} pidArray={pidArray}
                            disabled={updateFiledDisabled} form={form}
                            addEditPName={setPName}
                />
                <Form.Item
                    name='name'
                    label='任务名称'
                    rules={[{required: true, message: '任务名称不能为空'}]}
                    disabled={updateFiledDisabled}
                >
                    <Input onChange={console.log} placeholder='任务名称'/>
                </Form.Item>
                <Form.Item name='description' label='任务描述' disabled={updateFiledDisabled}>
                    <TextArea
                        placeholder='请输入任务描述'
                        maxLength={100}
                        rows={2}
                        showCount
                    />
                </Form.Item>
                <Form.Item name='state' initialValue='8' label='任务状态'
                           rules={[{required: true, message: '任务状态不能为空'}]} disabled={updateFiledDisabled}>
                    <Radio.Group>
                        <Space direction='vertical'>
                            {/*<Radio value='8'><Tag color='primary'>未开始</Tag></Radio>*/}
                            {/*<Radio value='9'><Tag color='warning'>进行中</Tag></Radio>*/}
                            {/*<Radio value='7'><Tag color='success'>已完成</Tag></Radio>*/}
                            {/*<Radio value='10'><Tag color='danger'>已逾期</Tag></Radio>*/}
                            {
                                stateList.map(stateDict =>
                                    <Radio key={stateDict.itemCode} value={stateDict.itemCode}>
                                        <Tag key={stateDict.itemCode}
                                             color={stateDict.jsonValue?.color}>{stateDict.itemName}</Tag>
                                    </Radio>
                                )
                            }
                        </Space>
                    </Radio.Group>
                </Form.Item>
                <Form.Item name='priority' label='任务优先级' initialValue='2'
                           rules={[{required: true, message: '任务优先级不能为空'}]} disabled={updateFiledDisabled}>
                    <Radio.Group>
                        <Space direction='vertical'>
                            {/*<Radio value='3'><Tag color='danger'>紧急重要</Tag></Radio>*/}
                            {/*<Radio value='2'><Tag color='warning'>不紧急重要</Tag></Radio>*/}
                            {/*<Radio value='1'><Tag>紧急不重要</Tag></Radio>*/}
                            {/*<Radio value='0'><Tag color='success'>不紧急不重要</Tag></Radio>*/}
                            {
                                priorityList.map(stateDict =>
                                    <Radio key={stateDict.itemCode} value={stateDict.itemCode}>
                                        <Tag key={stateDict.itemCode}
                                             color={stateDict.jsonValue?.color}>{stateDict.itemName}</Tag>
                                    </Radio>
                                )
                            }
                        </Space>
                    </Radio.Group>
                </Form.Item>
                <DataPickItemPopup disabled={updateFiledDisabled} fieldName={"expectedStartTime"}
                                   labelName={"预计开始时间"}/>
                <DataPickItemPopup disabled={updateFiledDisabled} fieldName={"expectedEndTime"}
                                   labelName={"预计结束时间"}/>
                <DataPickItemPopup disabled={updateFiledDisabled} fieldName={"actualStartTime"}
                                   labelName={"实际开始时间"}/>
                <DataPickItemPopup disabled={updateFiledDisabled} fieldName={"actualEndTime"}
                                   labelName={"实际结束时间"}/>
            </Form>
        </>
    )
}
export default DetailForm;