import {ActionSheet, Button, Dialog, Divider, Tabs, TextArea, Toast} from "antd-mobile";
import {Fragment, useEffect, useMemo, useRef, useState} from "react";
import "./index.css"
import {useOutletContext, useSearchParams} from "react-router-dom";
import {addTaskLog, listTaskLog} from "../../api/detailLogTaskApi";
import dayjs from "dayjs";
import {copyTextToClipboard} from "../../utils/copyToClipBoard";

export function DetailLogTask() {
    // 设置标题栏
    const {setTitle, setRightDesc} = useOutletContext();
    useEffect(() => {
        setTitle("日记/心得")
        setRightDesc(<Fragment/>);
    }, [])
    let [params] = useSearchParams();

    // 发送展示日记
    const [currentShow, setCurrentShow] = useState('need');
    const textAreaRef = useRef(null);
    const [taskLogList, setTaskLogList] = useState([]);
    const [sendValue, setSendValue] = useState('');

    // 点击操作面板
    const [actionSheetVisible, setActionSheetVisible] = useState(false)
    const [currentTask, setCurrentTask] = useState({});
    const [actions,setActions] = useState([
        { text: '复制', key: 'copy' },
        { text: '有效', key: 'need' },
        { text: '创建任务', key: 'addTask'},
        {
            text: '删除',
            key: 'delete',
            onClick: async () => {
                const result = await Dialog.confirm({ content: '确定要删除吗？' })
                if (result) {
                    Toast.show('执行了删除操作')
                }
            },
        },
    ]);

    const taskLogMapMemory = useMemo(() => {
        return taskLogList.reduce((map, taskLog) => {
            if (!map.has(dayjs(taskLog.createdDate).format("YYYY-MM-DD"))) {
                map.set(dayjs(taskLog.createdDate).format("YYYY-MM-DD"), []);
            }
            map.get(dayjs(taskLog.createdDate).format("YYYY-MM-DD"))?.push(taskLog);
            return map;
        }, new Map());
    }, [taskLogList,currentShow])
    const handleSend = () => {
        addTaskLog({
            "description": sendValue,
            "taskId": params.get('id'),
        }).then(res => {
            // 加入当前集合中
            setTaskLogList([...taskLogList,res]);
        })
        setSendValue('')
        textAreaRef.current.focus();
    };
    useEffect(() => {
        textAreaRef.current.focus();
        // 获取之前的日志信息，根据日期分组排序，遍历map
        listTaskLog(`{
            "sortList":[{"direction":"DESC","property":"createdDate"}],
            "data": {
                "andList":[{
                    "name":"taskId",
                    "operateType":"=",
                    "value":"${params.get('id')}"
                }]
            }
        }`).then(res => {
            console.log({res})
            if (res.content.length > 0) {
                setTaskLogList(res.content.reverse());
                // setTaskLogMap(res.content.reduce((map, taskLog) => {
                //     if (!map.has(dayjs(taskLog.createdDate).format("YYYY-MM-DD"))) {
                //         map.set(dayjs(taskLog.createdDate).format("YYYY-MM-DD"), []);
                //     }
                //     map.get(dayjs(taskLog.createdDate).format("YYYY-MM-DD"))?.push(taskLog);
                //     return map;
                // }, new Map()));
            }
        })
    }, [])
    function NoNeed(){
        return <s>失效</s>
    }
    return <Fragment>
        <div className="log-task-detail">
            <div className="log-detail">
                <Tabs defaultActiveKey='need' onChange={(key)=>setCurrentShow(key)}>
                    <Tabs.Tab title='全部' key='all' />
                    <Tabs.Tab title='有效' key='need' />
                    <Tabs.Tab key='noneed' title=<NoNeed/> />
                </Tabs>
                {
                    Array.from(taskLogMapMemory.keys()).map(key => {
                        return (<Fragment>
                            {dayjs().isSame(dayjs(key), "date") ?
                                <div key={key} style={{width: "100%", padding: "0 40px"}}>
                                    <Divider>今日</Divider>
                                </div> :
                                <div key={key} style={{width: "100%", padding: "0 20px"}}>
                                    <Divider>{key}</Divider>
                                </div>}
                            {taskLogMapMemory.get(key).map(taskLog => {
                                return <div key={taskLog.id} style={{ display: 'flex', width: "80%", padding: "2px 20px"}}>
                                    <span className="detail-line" onClick={()=>{
                                        actions[1]={ text: '失效', key: 'noneed' }
                                        // setActions([...actions])
                                        setCurrentTask(taskLog)
                                        setActionSheetVisible(true)
                                    }}>
                                        {taskLog.description}</span>
                                </div>
                            })}</Fragment>)
                    })
                }
            </div>

            <div className="log-send">
                <TextArea
                    ref={(node) => {
                        textAreaRef.current = node;
                    }}
                    value={sendValue}
                    onChange={(val) => setSendValue(val)}
                    placeholder='输入日记心得，长按日记心得有惊喜'
                    rows={1}
                    autoSize={{minRows: 1, maxRows: 5}}
                    maxLength={100}
                />
                <Button
                    className="send-button"
                    onClick={handleSend}
                >发送
                </Button>
                <ActionSheet
                    extra='请选择你要进行的操作'
                    cancelText='取消'
                    visible={actionSheetVisible}
                    actions={actions}
                    onClose={() => setActionSheetVisible(false)}
                    onAction={action => {
                        if (action.key === 'copy') {
                            copyTextToClipboard(currentTask.description)
                        }
                        setActionSheetVisible(false)
                    }}
                />
            </div>
        </div>

    </Fragment>

}